// Java Authentication System
// ContentParser Class

// Build vector of Jar information file entries

import java.io.*;
import java.util.*;

public class ContentParser {
	
	ContentParser() {
	}

	///////////////////////////////////////////////
	// State table
	///////////////////////////////////////////////
	// State			In				Next State
	///////////////////////////////////////////////
	//	1 NAME			-, _, A-Z, a-z, 0-9	1 NAME
	//	1 NAME			0x3A	:			2 SEPARATOR

	//	2 SEPARATOR		0x20	SP			3 VALUE

	//	3 VALUE			Unicode				3 VALUE
	//	3 VALUE			0x0D	CR			4 CR1
	//	3 VALUE			0x0A	LF			5 LF

	//	4 CRState		-, _,A-Z, a-z, 0-9	1 NAME	(New Line)
	//	4 CRState		0x20	SP			3 VALUE (Continuation)
	//	4 CRState		0x0D	CR			8 NEWSECTION
	//	4 CRState		0x0A	LF			6 CRLFState
	

	//	5 LFState		-, _,A-Z, a-z, 0-9	1 NAME
	//	5 LFState		0x20	SP			3 VALUE
	//	5 LFState		0x0A	LF			8 NEWSECTION

	//	6 CRLFState		-, _,A-Z, a-z, 0-9	1 NAME
	//	6 CRLFState		0x20	SP			3 VALUE
	//	6 CRLFState		0x0D	CR			7 CRLFCRState
	

	//	7 CRLFCRState	0x0A	LF			8 NEWSECTION

	//	8 NEWSECTION	-,_,A-Z, a-z, 0-9	1 NAME

	public Vector parse( InputStream instream, int size ) {
		Vector entries = new Vector(10);
		DataInputStream in = new DataInputStream( instream );
		int state = 8;	// Init state is NewSection
		int prevstate = 0;	// Previous state 
		int numOfDigests=0;
		ContentSection section = null;
		byte[] value= new byte[70];	// (XXX) This is below standard limits
		int valuelen = 0;
		byte[] headername = new byte[70];
		int headerlen = 0;	
		byte[] sectiondata = new byte[256]; // (XXX) An arbitrary value
		int sectionlen = 0;
		boolean savesection = false;
		byte ch = 0;
		boolean gotname = false; // Used to get around Netscapes Digest feature

		for( int i=0; i < size; i++ ) {
			try {
				ch = (byte) in.readUnsignedByte();
			} catch ( IOException e ) {
				System.err.println("JAS Error:	IO Exception occurred");
			}
			
			switch ( state ) {
				case 1:	// Name

					if (	(( ch >= 0x30 )	&& ( ch <= 0x39 ))	||		// 0-9
							(( ch >= 0x41 )	&& ( ch <= 0x5A ))	||		// A-Z
							(( ch >= 0x61 )	&& ( ch <= 0x7A ))	|| 		// a-z
							( ch == 0x5F )	|| ( ch == 0x2D)	)		// _, -
					{	
						state = 1;
					} else if ( ch == 0x3A ) {
						state = 2;
					} else {
						System.err.println("JAS Error:	Encountered an unexpected character while section was parsed");
						System.exit(1);
					}
					prevstate = 1;
					break;
				case 2:	// Separator
					if ( ch == 0x20 ) {
						state = 3;
					} else {
						System.err.println("JAS Error:	Encountered an unexpected character while section was parsed");
						System.exit(1);
					}		
					prevstate = 2;
					break;
				case 3:	// Value
					if ( (ch >= 0x20) && (ch <= 0x80) ) {		// XXX Not Unicode 
						state = 3;
					} else if ( ch == 0x0D ) {
						state = 4;
					} else if ( ch == 0x0A ) {
						state = 5;
					} else {
						System.err.println("JAS Error:	Encountered an unexpected character while section was parsed");
						System.exit(1);
					}
					prevstate = 3;
					break;
				case 4:	// CRState
					if (	(( ch >= 0x30 )	&& ( ch <= 0x39 ))	||		// 0-9
							(( ch >= 0x41 )	&& ( ch <= 0x5A ))	||		// A-Z
							(( ch >= 0x61 )	&& ( ch <= 0x7A ))	|| 		// a-z
							( ch == 0x5F )	|| ( ch == 0x2D)	)		// _, -
					{		
						state = 1;
					} else if( ch == 0x0D ) {
						state = 8; 
					} else if ( ch == 0x0A ) {
						state = 6;
					} else if ( ch == 0x20 ) {
						state = 3;
					} else {
						System.err.println("JAS Error:	Encountered an unexpected character while section was parsed");
						System.exit(1);
					}
					prevstate = 4;
					break;
				case 5:	// LFState
					if (	(( ch >= 0x30 )	&& ( ch <= 0x39 ))	||		// 0-9
							(( ch >= 0x41 )	&& ( ch <= 0x5A ))	||		// A-Z
							(( ch >= 0x61 )	&& ( ch <= 0x7A ))	|| 		// a-z
							( ch == 0x5F )	|| ( ch == 0x2D)	)		// _, -
					{		
						state = 1;
					} else if ( ch == 0x0A ) {
						state = 8;
					} else if ( ch == 0x20 ) {
						state = 3;
					} else {
						System.err.println("JAS Error:	Encountered an unexpected character while section was parsed");
						System.exit(1);
					}
					prevstate = 5;
					break;
				case 6: // CRLFState
					if (	(( ch >= 0x30 )	&& ( ch <= 0x39 ))	||		// 0-9
							(( ch >= 0x41 )	&& ( ch <= 0x5A ))	||		// A-Z
							(( ch >= 0x61 )	&& ( ch <= 0x7A ))	|| 		// a-z
							( ch == 0x5F )	|| ( ch == 0x2D)	)		// _, -
					{		
						state = 1;
					} else if ( ch == 0x0D ) {
						state = 7;
					} else if ( ch == 0x20 ) {
						state = 3;
					} else {
						System.err.println("JAS Error:	Encountered an unexpected character while section was parsed");
						System.exit(1);
					}
					prevstate = 6;
					break;
				case 7:	// CRLFCRState
					if ( ch == 0x0A ) {
						state = 8;
					} else {
						System.err.println("JAS Error:	Encountered an unexpected character while section was parsed");
						System.exit(1);
					}
					prevstate = 7;
					break;
				case 8: // NewSection
					if (	(( ch >= 0x30 )	&& ( ch <= 0x39 ))	||		// 0-9
							(( ch >= 0x41 )	&& ( ch <= 0x5A ))	||		// A-Z
							(( ch >= 0x61 )	&& ( ch <= 0x7A ))	|| 		// a-z
							( ch == 0x5F )	|| ( ch == 0x2D)	)		// _, -
					{			
						state = 1;
						// JasDebug.println("New Section --------------------------");
					} else {
						System.err.println("JAS Error:	Encountered an unexpected character while section was parsed");
						System.exit(1);
					}
					prevstate = 8;
					break;
				default:
					System.err.println("JAS Error:	Unreachable state");
					System.exit(1);
			}
			// JasDebug.println("PrSt:	" + prevstate + "	Ch:	" + (int)ch + "	St:	" + state);

			//////////////////		
			// State events
			//////////////////
			sectiondata[sectionlen] = ch;
			sectionlen++;

			if ( state == 1 ) {
				headername[headerlen] = ch;
				headerlen++;
			}

			if ( state == 2 ) {
				value = new byte[70];
				valuelen=0;
			}

			if ( state == 3 ) {
				if ( prevstate != 2 ) {
					value[valuelen] = ch;
					valuelen++;
				}
			}

			if ( (state == 4) || (state == 5) ) {
				String header = new String( headername );
				header = header.trim();				

				if ( header.equals("Name") ) {
					gotname = true;
					section = new ContentSection();
					section.name = new byte[valuelen];
					System.arraycopy( value, 0, section.name, 0, valuelen );
					// JasDebug.println(">" + section.getName() + "<");
				}
				if ( (header.endsWith("-Digest")) && (gotname == true) ) {
					int ix = header.indexOf('-');
					String filur = header.substring(0, ix);
					section.digests[ numOfDigests ].algorithm = filur;
					ix=0;
					section.digests[ numOfDigests ].digest = new byte[valuelen];
					System.arraycopy( value, 0, section.digests[ numOfDigests ].digest, 0, valuelen );
					numOfDigests++;
				}



				headername = new byte[70];
				headerlen=0;
			}
	
			// Prepare next section
			if ( state == 8 ) {
				headername = new byte[70];
				headerlen=0;
				numOfDigests=0;
				gotname = false;

				// If we got section data we care about
				if (section != null) {
					int unwanted = 0;
					// Just one character represents a newline
					if ( ( prevstate == 4 ) || ( prevstate == 5 ) ) {
						// Remove that character
						unwanted = 1; 	
					}

					// Two characters represents a newline
					if ( prevstate == 7 ) {
						// Remove those characters
						unwanted = 2;
					}
					section.rawdata = new byte[sectionlen - unwanted];
					System.arraycopy( sectiondata, 0, section.rawdata, 0, sectionlen - unwanted);
					entries.addElement(section);
				}
				sectiondata = new byte[512]; // (XXX) Arbitrary value
				sectionlen = 0;
			}
		}
		return entries;
	}
}
