// Java Authentication System
// SignatureFile class

import java.util.zip.*;
import java.util.*;
import java.io.*;
import java.security.*;
import sun.misc.BASE64Decoder;

public class SignatureFile {
	private Vector entries = new Vector(10);
	
	SignatureFile( InputStream in, int size ) {
		JasDebug.println("SignatureFile object created");
		ContentParser p = new ContentParser();
		entries = p.parse( in, size );
	}

	public Vector getContent() {
		return entries;
	}

	public boolean checkManifestIntegrity(ManifestFile manifest) {

		// For each entry in the manifest file:
		//		1. Find the corresponding file in the archive.
		//
		//		2. Compute a digest for the file and compare
		//		to the digest stored in the manifest file

		Vector mfv = manifest.getContent();
		ContentSection sfentry = null;
		BASE64Decoder decoder = new BASE64Decoder();
		int passed = 0;		// Number of files that passed the integrity check

		JasDebug.println("Starting manifest integrity check");

		for (Enumeration sfes = entries.elements() ; sfes.hasMoreElements() ;) {
           	sfentry = (ContentSection) sfes.nextElement();
			String name = sfentry.getName();
			DigestData[] digests = sfentry.getDigests();
			
			// Get the corresponding manifest entry

			JasDebug.println("Now checking: " + name);
			for (Enumeration mfs = mfv.elements() ; mfs.hasMoreElements() ;) {
           		ContentSection manifestentry = (ContentSection)	mfs.nextElement();
				
				if( name.equals(manifestentry.getName()) ) {
					JasDebug.println("Found a file that matches");
					try {	
						// Check if any of the stored hashes is of a supported type
						for( int i=0; i < digests.length; i++) {
							if ( (JasConfig.supportedHashAlgorithm( digests[i].algorithm )) == true ) {
								JasDebug.println(digests[i].algorithm + " is a supported algorithm");					
								MessageDigest md = MessageDigest.getInstance( digests[i].algorithm );
								md.update( manifestentry.rawdata );
								byte[] digest = md.digest();
								String digestString = new String( digest );
								byte[] manifestdigest = decoder.decodeBuffer( digestString );
								digestString = new String( digest );
								byte[] sigfiledigest = decoder.decodeBuffer( digestString );
								if ( (MessageDigest.isEqual( sigfiledigest, manifestdigest )) == true ) {
									JasDebug.println("Digests match!");
									passed++;
									break;
								}							
							}
						}
					} catch ( NoSuchAlgorithmException e ) {
						JasDebug.println("NoSuchAlgorithmException");
						return false;
					} catch ( IOException e ) {
						JasDebug.println("NoSuchAlgorithmException");
						return false;
					}
					break;
				}
			}
		}
	
		if ( passed == entries.size() )
			return true;
		else
			return false;
	}
}
